@@ -1,35 +1,31 @@
 /* -------------------------------------------------------------------------
  * pg_dumplo
  *
- * $Header: /cvsroot/pgsql-server/contrib/pg_dumplo/main.c,v 1.10 2001/11/12 17:44:14 momjian Exp $
+ * $Header: /cvsroot/pgsql/contrib/pg_dumplo/Attic/main.c,v 1.18 2003/08/07 21:11:57 tgl Exp $
  *
  *					Karel Zak 1999-2000
  * -------------------------------------------------------------------------
  */
 
-/* We import postgres_fe.h mostly to get the HAVE_GETOPT_LONG configure result. */
-#ifndef OUT_OF_PG
 #include "postgres_fe.h"
-#endif
 
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <string.h>
 #include <errno.h>
+#include <unistd.h>
 
-#include <libpq-fe.h>
-#include <libpq/libpq-fs.h>
+#include "libpq-fe.h"
+#include "libpq/libpq-fs.h"
 
 #include "pg_dumplo.h"
 
-#ifdef HAVE_GETOPT_LONG
-#include <getopt.h>
-#define no_argument 0
-#define required_argument 1
+#ifndef HAVE_STRDUP
+#include "strdup.h"
 #endif
 
-extern int	errno;
+#include "getopt_long.h"
+
+#ifndef HAVE_OPTRESET
+int			optreset;
+#endif
 
 char	   *progname = NULL;
 
@@ -73,8 +69,6 @@ main(int argc, char **argv)
 	{
 		int			arg;
 		extern int	optind;
-
-#ifdef HAVE_GETOPT_LONG
 		int			l_index = 0;
 		static struct option l_opt[] = {
 			{"help", no_argument, 0, 'h'},
@@ -95,10 +89,6 @@ main(int argc, char **argv)
 
 		while ((arg = getopt_long(argc, argv, "?aeho:u:p:qd:l:t:irs:w", l_opt, &l_index)) != -1)
 		{
-#else
-		while ((arg = getopt(argc, argv, "?aeho:u:p:qd:l:t:irs:w")) != -1)
-		{
-#endif
 			switch (arg)
 			{
 				case '?':
@@ -197,6 +187,8 @@ main(int argc, char **argv)
 	if (pgLO->action != ACTION_SHOW)
 		index_file(pgLO);
 
+	PQexec(pgLO->conn, "SET search_path = public");
+
 	PQexec(pgLO->conn, "BEGIN");
 
 	switch (pgLO->action)
@@ -274,17 +266,14 @@ parse_lolist(LODumpMaster * pgLO)
 static void
 usage()
 {
-	printf("\npg_dumplo %s - PostgreSQL large objects dump\n", VERSION);
+	printf("\npg_dumplo %s - PostgreSQL large objects dump\n", PG_VERSION);
 	puts("pg_dumplo [option]\n\n"
-
-#ifdef HAVE_GETOPT_LONG
-
 		 "-h --help                    this help\n"
 	   "-u --user=<username>         username for connection to server\n"
 	   "-p --password=<password>     password for connection to server\n"
 		 "-d --db=<database>           database name\n"
 		 "-t --host=<hostname>         server hostname\n"
-		 "-o --port=<port>             database server port (default: 5432)\n" 
+	"-o --port=<port>             database server port (default: 5432)\n"
 		 "-s --space=<dir>             directory with dump tree (for export/import)\n"
 		 "-i --import                  import large obj dump tree to DB\n"
 	"-e --export                  export (dump) large obj to dump tree\n"
@@ -293,27 +282,7 @@ usage()
 		 "-r --remove                  if is set '-i' try remove old LO\n"
 		 "-q --quiet                   run quietly\n"
 		 "-w --show                    not dump, but show all LO in DB\n"
-		);						/* puts() */
-
-#else
-		 "-h                           this help\n"
-	   "-u <username>                username for connection to server\n"
-	   "-p <password>                password for connection to server\n"
-		 "-d <database>                database name\n"
-		 "-t <hostname>                server hostname\n"
-		 "-o <port>                    database server port (default: 5432)\n" 
-		 "-s <dir>                     directory with dump tree (for export/import)\n"
-		 "-i                           import large obj dump tree to DB\n"
-	"-e                           export (dump) large obj to dump tree\n"
-		 "-l <table.attr ...>          dump attribute (columns) with LO to dump tree\n"
-		 "-a                           dump all LO in DB (default)\n"
-		 "-r                           if is set '-i' try remove old LO\n"
-		 "-q                           run quietly\n"
-		 "-w                           not dump, but show all LO in DB\n"
-		);						/* puts() */
-#endif
-
-	puts(
+		 "\n"
 		 "Example (dump):   pg_dumplo -d my_db -s /my_dump/dir -l t1.a t1.b t2.a\n"
 		 "                  pg_dumplo -a -d my_db -s /my_dump/dir\n"
 		 "Example (import): pg_dumplo -i -d my_db -s /my_dump/dir\n"
@@ -321,6 +290,6 @@ usage()
 		 "Note:  * option '-l' must be last option!\n"
 	"       * option '-i' without option '-r' make new large obj in DB\n"
 		 "         not rewrite old, the '-i' UPDATE oid numbers in table.attr only!\n"
-		 "       * if is not set option -s, the pg_dumplo use $PWD\n"
+		 "       * if option -s is not set, pg_dumplo uses $PWD\n"
 		);						/* puts() */
 }
