@@ -7,22 +7,25 @@
  * detection and resolution algorithms.
  *
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/storage/lmgr/deadlock.c,v 1.8 2001/10/28 06:25:50 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/storage/lmgr/deadlock.c,v 1.25 2003/09/25 06:58:02 petere Exp $
  *
  *	Interface:
  *
  *	DeadLockCheck()
+ *	DeadLockReport()
+ *	RememberSimpleDeadLock()
  *	InitDeadLockChecking()
  *
  *-------------------------------------------------------------------------
  */
 #include "postgres.h"
 
+#include "lib/stringinfo.h"
 #include "miscadmin.h"
 #include "storage/proc.h"
 #include "utils/memutils.h"
@@ -31,8 +34,8 @@
 /* One edge in the waits-for graph */
 typedef struct
 {
-	PROC	   *waiter;			/* the waiting process */
-	PROC	   *blocker;		/* the process it is waiting for */
+	PGPROC	   *waiter;			/* the waiting process */
+	PGPROC	   *blocker;		/* the process it is waiting for */
 	int			pred;			/* workspace for TopoSort */
 	int			link;			/* workspace for TopoSort */
 } EDGE;
@@ -41,20 +44,35 @@ typedef struct
 typedef struct
 {
 	LOCK	   *lock;			/* the lock whose wait queue is described */
-	PROC	  **procs;			/* array of PROC *'s in new wait order */
+	PGPROC	  **procs;			/* array of PGPROC *'s in new wait order */
 	int			nProcs;
 } WAIT_ORDER;
 
+/*
+ * Information saved about each edge in a detected deadlock cycle.	This
+ * is used to print a diagnostic message upon failure.
+ *
+ * Note: because we want to examine this info after releasing the LockMgrLock,
+ * we can't just store LOCK and PGPROC pointers; we must extract out all the
+ * info we want to be able to print.
+ */
+typedef struct
+{
+	LOCKTAG		locktag;		/* ID of awaited lock object */
+	LOCKMODE	lockmode;		/* type of lock we're waiting for */
+	int			pid;			/* PID of blocked backend */
+} DEADLOCK_INFO;
 
-static bool DeadLockCheckRecurse(PROC *proc);
-static bool TestConfiguration(PROC *startProc);
-static bool FindLockCycle(PROC *checkProc,
+
+static bool DeadLockCheckRecurse(PGPROC *proc);
+static int	TestConfiguration(PGPROC *startProc);
+static bool FindLockCycle(PGPROC *checkProc,
 			  EDGE *softEdges, int *nSoftEdges);
-static bool FindLockCycleRecurse(PROC *checkProc,
+static bool FindLockCycleRecurse(PGPROC *checkProc, int depth,
 					 EDGE *softEdges, int *nSoftEdges);
 static bool ExpandConstraints(EDGE *constraints, int nConstraints);
 static bool TopoSort(LOCK *lock, EDGE *constraints, int nConstraints,
-		 PROC **ordering);
+		 PGPROC **ordering);
 
 #ifdef DEBUG_DEADLOCK
 static void PrintLockQueue(LOCK *lock, const char *info);
@@ -66,18 +84,18 @@ static void PrintLockQueue(LOCK *lock, const char *info);
  */
 
 /* Workspace for FindLockCycle */
-static PROC **visitedProcs;		/* Array of visited procs */
+static PGPROC **visitedProcs;	/* Array of visited procs */
 static int	nVisitedProcs;
 
 /* Workspace for TopoSort */
-static PROC **topoProcs;		/* Array of not-yet-output procs */
+static PGPROC **topoProcs;		/* Array of not-yet-output procs */
 static int *beforeConstraints;	/* Counts of remaining before-constraints */
 static int *afterConstraints;	/* List head for after-constraints */
 
 /* Output area for ExpandConstraints */
 static WAIT_ORDER *waitOrders;	/* Array of proposed queue rearrangements */
 static int	nWaitOrders;
-static PROC **waitOrderProcs;	/* Space for waitOrders queue contents */
+static PGPROC **waitOrderProcs; /* Space for waitOrders queue contents */
 
 /* Current list of constraints being considered */
 static EDGE *curConstraints;
@@ -88,6 +106,8 @@ static int	maxCurConstraints;
 static EDGE *possibleConstraints;
 static int	nPossibleConstraints;
 static int	maxPossibleConstraints;
+static DEADLOCK_INFO *deadlockDetails;
+static int	nDeadlockDetails;
 
 
 /*
@@ -110,8 +130,10 @@ InitDeadLockChecking(void)
 
 	/*
 	 * FindLockCycle needs at most MaxBackends entries in visitedProcs[]
+	 * and deadlockDetails[].
 	 */
-	visitedProcs = (PROC **) palloc(MaxBackends * sizeof(PROC *));
+	visitedProcs = (PGPROC **) palloc(MaxBackends * sizeof(PGPROC *));
+	deadlockDetails = (DEADLOCK_INFO *) palloc(MaxBackends * sizeof(DEADLOCK_INFO));
 
 	/*
 	 * TopoSort needs to consider at most MaxBackends wait-queue entries,
@@ -125,10 +147,12 @@ InitDeadLockChecking(void)
 	 * We need to consider rearranging at most MaxBackends/2 wait queues
 	 * (since it takes at least two waiters in a queue to create a soft
 	 * edge), and the expanded form of the wait queues can't involve more
-	 * than MaxBackends total waiters.
+	 * than MaxBackends total waiters.	(But avoid palloc(0) if
+	 * MaxBackends = 1.)
 	 */
-	waitOrders = (WAIT_ORDER *) palloc((MaxBackends / 2) * sizeof(WAIT_ORDER));
-	waitOrderProcs = (PROC **) palloc(MaxBackends * sizeof(PROC *));
+	waitOrders = (WAIT_ORDER *)
+		palloc(((MaxBackends + 1) / 2) * sizeof(WAIT_ORDER));
+	waitOrderProcs = (PGPROC **) palloc(MaxBackends * sizeof(PGPROC *));
 
 	/*
 	 * Allow at most MaxBackends distinct constraints in a configuration.
@@ -170,13 +194,17 @@ InitDeadLockChecking(void)
  * only look at regular locks.
  *
  * We must have already locked the master lock before being called.
- * NOTE: although the lockctl structure appears to allow each lock
- * table to have a different LWLock, all locks that can block had
+ * NOTE: although the lockmethod structure appears to allow each lock
+ * table to have a different masterLock, all locks that can block had
  * better use the same LWLock, else this code will not be adequately
  * interlocked!
+ *
+ * On failure, deadlock details are recorded in deadlockDetails[] for
+ * subsequent printing by DeadLockReport().  That activity is separate
+ * because we don't want to do it while holding the master lock.
  */
 bool
-DeadLockCheck(PROC *proc)
+DeadLockCheck(PGPROC *proc)
 {
 	int			i,
 				j;
@@ -188,13 +216,25 @@ DeadLockCheck(PROC *proc)
 
 	/* Search for deadlocks and possible fixes */
 	if (DeadLockCheckRecurse(proc))
+	{
+		/*
+		 * Call FindLockCycle one more time, to record the correct
+		 * deadlockDetails[] for the basic state with no rearrangements.
+		 */
+		int			nSoftEdges;
+
+		nWaitOrders = 0;
+		if (!FindLockCycle(proc, possibleConstraints, &nSoftEdges))
+			elog(FATAL, "deadlock seems to have disappeared");
+
 		return true;			/* cannot find a non-deadlocked state */
+	}
 
 	/* Apply any needed rearrangements of wait queues */
 	for (i = 0; i < nWaitOrders; i++)
 	{
 		LOCK	   *lock = waitOrders[i].lock;
-		PROC	  **procs = waitOrders[i].procs;
+		PGPROC	  **procs = waitOrders[i].procs;
 		int			nProcs = waitOrders[i].nProcs;
 		PROC_QUEUE *waitQueue = &(lock->waitProcs);
 
@@ -234,7 +274,7 @@ DeadLockCheck(PROC *proc)
  * rearrangements of lock wait queues (if any).
  */
 static bool
-DeadLockCheckRecurse(PROC *proc)
+DeadLockCheckRecurse(PGPROC *proc)
 {
 	int			nEdges;
 	int			oldPossibleConstraints;
@@ -270,7 +310,7 @@ DeadLockCheckRecurse(PROC *proc)
 		{
 			/* Regenerate the list of possible added constraints */
 			if (nEdges != TestConfiguration(proc))
-				elog(FATAL, "DeadLockCheckRecurse: inconsistent results");
+				elog(FATAL, "inconsistent results during deadlock check");
 		}
 		curConstraints[nCurConstraints] =
 			possibleConstraints[oldPossibleConstraints + i];
@@ -299,8 +339,8 @@ DeadLockCheckRecurse(PROC *proc)
  * number of soft edges.
  *--------------------
  */
-static bool
-TestConfiguration(PROC *startProc)
+static int
+TestConfiguration(PGPROC *startProc)
 {
 	int			softFound = 0;
 	EDGE	   *softEdges = possibleConstraints + nPossibleConstraints;
@@ -355,9 +395,12 @@ TestConfiguration(PROC *startProc)
  *
  * Scan outward from the given proc to see if there is a cycle in the
  * waits-for graph that includes this proc.  Return TRUE if a cycle
- * is found, else FALSE.  If a cycle is found, we also return a list of
+ * is found, else FALSE.  If a cycle is found, we return a list of
  * the "soft edges", if any, included in the cycle.  These edges could
- * potentially be eliminated by rearranging wait queues.
+ * potentially be eliminated by rearranging wait queues.  We also fill
+ * deadlockDetails[] with information about the detected cycle; this info
+ * is not used by the deadlock algorithm itself, only to print a useful
+ * message after failing.
  *
  * Since we need to be able to check hypothetical configurations that would
  * exist after wait queue rearrangement, the routine pays attention to the
@@ -365,26 +408,27 @@ TestConfiguration(PROC *startProc)
  * be believed in preference to the actual ordering seen in the locktable.
  */
 static bool
-FindLockCycle(PROC *checkProc,
+FindLockCycle(PGPROC *checkProc,
 			  EDGE *softEdges,	/* output argument */
 			  int *nSoftEdges)	/* output argument */
 {
 	nVisitedProcs = 0;
+	nDeadlockDetails = 0;
 	*nSoftEdges = 0;
-	return FindLockCycleRecurse(checkProc, softEdges, nSoftEdges);
+	return FindLockCycleRecurse(checkProc, 0, softEdges, nSoftEdges);
 }
 
 static bool
-FindLockCycleRecurse(PROC *checkProc,
+FindLockCycleRecurse(PGPROC *checkProc,
+					 int depth,
 					 EDGE *softEdges,	/* output argument */
 					 int *nSoftEdges)	/* output argument */
 {
-	PROC	   *proc;
+	PGPROC	   *proc;
 	LOCK	   *lock;
-	HOLDER	   *holder;
+	PROCLOCK   *proclock;
 	SHM_QUEUE  *lockHolders;
 	LOCKMETHODTABLE *lockMethodTable;
-	LOCKMETHODCTL *lockctl;
 	PROC_QUEUE *waitQueue;
 	int			queue_size;
 	int			conflictMask;
@@ -401,7 +445,16 @@ FindLockCycleRecurse(PROC *checkProc,
 		{
 			/* If we return to starting point, we have a deadlock cycle */
 			if (i == 0)
+			{
+				/*
+				 * record total length of cycle --- outer levels will now
+				 * fill deadlockDetails[]
+				 */
+				Assert(depth <= MaxBackends);
+				nDeadlockDetails = depth;
+
 				return true;
+			}
 
 			/*
 			 * Otherwise, we have a cycle but it does not include the
@@ -423,9 +476,8 @@ FindLockCycleRecurse(PROC *checkProc,
 	if (lock == NULL)
 		return false;
 	lockMethodTable = GetLocksMethodTable(lock);
-	lockctl = lockMethodTable->ctl;
-	numLockModes = lockctl->numLockModes;
-	conflictMask = lockctl->conflictTab[checkProc->waitLockMode];
+	numLockModes = lockMethodTable->numLockModes;
+	conflictMask = lockMethodTable->conflictTab[checkProc->waitLockMode];
 
 	/*
 	 * Scan for procs that already hold conflicting locks.	These are
@@ -433,32 +485,42 @@ FindLockCycleRecurse(PROC *checkProc,
 	 */
 	lockHolders = &(lock->lockHolders);
 
-	holder = (HOLDER *) SHMQueueNext(lockHolders, lockHolders,
-									 offsetof(HOLDER, lockLink));
+	proclock = (PROCLOCK *) SHMQueueNext(lockHolders, lockHolders,
+										 offsetof(PROCLOCK, lockLink));
 
-	while (holder)
+	while (proclock)
 	{
-		proc = (PROC *) MAKE_PTR(holder->tag.proc);
+		proc = (PGPROC *) MAKE_PTR(proclock->tag.proc);
 
 		/* A proc never blocks itself */
 		if (proc != checkProc)
 		{
 			for (lm = 1; lm <= numLockModes; lm++)
 			{
-				if (holder->holding[lm] > 0 &&
+				if (proclock->holding[lm] > 0 &&
 					((1 << lm) & conflictMask) != 0)
 				{
 					/* This proc hard-blocks checkProc */
-					if (FindLockCycleRecurse(proc, softEdges, nSoftEdges))
+					if (FindLockCycleRecurse(proc, depth + 1,
+											 softEdges, nSoftEdges))
+					{
+						/* fill deadlockDetails[] */
+						DEADLOCK_INFO *info = &deadlockDetails[depth];
+
+						info->locktag = lock->tag;
+						info->lockmode = checkProc->waitLockMode;
+						info->pid = checkProc->pid;
+
 						return true;
-					/* If no deadlock, we're done looking at this holder */
+					}
+					/* If no deadlock, we're done looking at this proclock */
 					break;
 				}
 			}
 		}
 
-		holder = (HOLDER *) SHMQueueNext(lockHolders, &holder->lockLink,
-										 offsetof(HOLDER, lockLink));
+		proclock = (PROCLOCK *) SHMQueueNext(lockHolders, &proclock->lockLink,
+										   offsetof(PROCLOCK, lockLink));
 	}
 
 	/*
@@ -480,7 +542,7 @@ FindLockCycleRecurse(PROC *checkProc,
 	if (i < nWaitOrders)
 	{
 		/* Use the given hypothetical wait queue order */
-		PROC	  **procs = waitOrders[i].procs;
+		PGPROC	  **procs = waitOrders[i].procs;
 
 		queue_size = waitOrders[i].nProcs;
 
@@ -496,8 +558,16 @@ FindLockCycleRecurse(PROC *checkProc,
 			if (((1 << proc->waitLockMode) & conflictMask) != 0)
 			{
 				/* This proc soft-blocks checkProc */
-				if (FindLockCycleRecurse(proc, softEdges, nSoftEdges))
+				if (FindLockCycleRecurse(proc, depth + 1,
+										 softEdges, nSoftEdges))
 				{
+					/* fill deadlockDetails[] */
+					DEADLOCK_INFO *info = &deadlockDetails[depth];
+
+					info->locktag = lock->tag;
+					info->lockmode = checkProc->waitLockMode;
+					info->pid = checkProc->pid;
+
 					/*
 					 * Add this edge to the list of soft edges in the
 					 * cycle
@@ -517,7 +587,7 @@ FindLockCycleRecurse(PROC *checkProc,
 		waitQueue = &(lock->waitProcs);
 		queue_size = waitQueue->size;
 
-		proc = (PROC *) MAKE_PTR(waitQueue->links.next);
+		proc = (PGPROC *) MAKE_PTR(waitQueue->links.next);
 
 		while (queue_size-- > 0)
 		{
@@ -529,8 +599,16 @@ FindLockCycleRecurse(PROC *checkProc,
 			if (((1 << proc->waitLockMode) & conflictMask) != 0)
 			{
 				/* This proc soft-blocks checkProc */
-				if (FindLockCycleRecurse(proc, softEdges, nSoftEdges))
+				if (FindLockCycleRecurse(proc, depth + 1,
+										 softEdges, nSoftEdges))
 				{
+					/* fill deadlockDetails[] */
+					DEADLOCK_INFO *info = &deadlockDetails[depth];
+
+					info->locktag = lock->tag;
+					info->lockmode = checkProc->waitLockMode;
+					info->pid = checkProc->pid;
+
 					/*
 					 * Add this edge to the list of soft edges in the
 					 * cycle
@@ -543,7 +621,7 @@ FindLockCycleRecurse(PROC *checkProc,
 				}
 			}
 
-			proc = (PROC *) MAKE_PTR(proc->links.next);
+			proc = (PGPROC *) MAKE_PTR(proc->links.next);
 		}
 	}
 
@@ -559,7 +637,7 @@ FindLockCycleRecurse(PROC *checkProc,
  *		specific new orderings for affected wait queues
  *
  * Input is a list of soft edges to be reversed.  The output is a list
- * of nWaitOrders WAIT_ORDER structs in waitOrders[], with PROC array
+ * of nWaitOrders WAIT_ORDER structs in waitOrders[], with PGPROC array
  * workspace in waitOrderProcs[].
  *
  * Returns TRUE if able to build an ordering that satisfies all the
@@ -582,7 +660,7 @@ ExpandConstraints(EDGE *constraints,
 	 */
 	for (i = nConstraints; --i >= 0;)
 	{
-		PROC	   *proc = constraints[i].waiter;
+		PGPROC	   *proc = constraints[i].waiter;
 		LOCK	   *lock = proc->waitLock;
 
 		/* Did we already make a list for this lock? */
@@ -628,7 +706,7 @@ ExpandConstraints(EDGE *constraints,
  * slowness of the algorithm won't really matter.
  *
  * The initial queue ordering is taken directly from the lock's wait queue.
- * The output is an array of PROC pointers, of length equal to the lock's
+ * The output is an array of PGPROC pointers, of length equal to the lock's
  * wait queue length (the caller is responsible for providing this space).
  * The partial order is specified by an array of EDGE structs.	Each EDGE
  * is one that we need to reverse, therefore the "waiter" must appear before
@@ -642,22 +720,22 @@ static bool
 TopoSort(LOCK *lock,
 		 EDGE *constraints,
 		 int nConstraints,
-		 PROC **ordering)		/* output argument */
+		 PGPROC **ordering)		/* output argument */
 {
 	PROC_QUEUE *waitQueue = &(lock->waitProcs);
 	int			queue_size = waitQueue->size;
-	PROC	   *proc;
+	PGPROC	   *proc;
 	int			i,
 				j,
 				k,
 				last;
 
 	/* First, fill topoProcs[] array with the procs in their current order */
-	proc = (PROC *) MAKE_PTR(waitQueue->links.next);
+	proc = (PGPROC *) MAKE_PTR(waitQueue->links.next);
 	for (i = 0; i < queue_size; i++)
 	{
 		topoProcs[i] = proc;
-		proc = (PROC *) MAKE_PTR(proc->links.next);
+		proc = (PGPROC *) MAKE_PTR(proc->links.next);
 	}
 
 	/*
@@ -743,18 +821,92 @@ PrintLockQueue(LOCK *lock, const char *info)
 {
 	PROC_QUEUE *waitQueue = &(lock->waitProcs);
 	int			queue_size = waitQueue->size;
-	PROC	   *proc;
+	PGPROC	   *proc;
 	int			i;
 
 	printf("%s lock %lx queue ", info, MAKE_OFFSET(lock));
-	proc = (PROC *) MAKE_PTR(waitQueue->links.next);
+	proc = (PGPROC *) MAKE_PTR(waitQueue->links.next);
 	for (i = 0; i < queue_size; i++)
 	{
 		printf(" %d", proc->pid);
-		proc = (PROC *) MAKE_PTR(proc->links.next);
+		proc = (PGPROC *) MAKE_PTR(proc->links.next);
 	}
 	printf("\n");
 	fflush(stdout);
 }
-
 #endif
+
+/*
+ * Report a detected deadlock, with available details.
+ */
+void
+DeadLockReport(void)
+{
+	StringInfoData buf;
+	int			i;
+
+	initStringInfo(&buf);
+	for (i = 0; i < nDeadlockDetails; i++)
+	{
+		DEADLOCK_INFO *info = &deadlockDetails[i];
+		int			nextpid;
+
+		/* The last proc waits for the first one... */
+		if (i < nDeadlockDetails - 1)
+			nextpid = info[1].pid;
+		else
+			nextpid = deadlockDetails[0].pid;
+
+		if (i > 0)
+			appendStringInfoChar(&buf, '\n');
+
+		if (info->locktag.relId == XactLockTableId && info->locktag.dbId == 0)
+		{
+			/* Lock is for transaction ID */
+			appendStringInfo(&buf,
+							 gettext("Process %d waits for %s on transaction %u; blocked by process %d."),
+							 info->pid,
+							 GetLockmodeName(info->lockmode),
+							 info->locktag.objId.xid,
+							 nextpid);
+		}
+		else
+		{
+			/* Lock is for a relation */
+			appendStringInfo(&buf,
+							 gettext("Process %d waits for %s on relation %u of database %u; blocked by process %d."),
+							 info->pid,
+							 GetLockmodeName(info->lockmode),
+							 info->locktag.relId,
+							 info->locktag.dbId,
+							 nextpid);
+		}
+	}
+	ereport(ERROR,
+			(errcode(ERRCODE_T_R_DEADLOCK_DETECTED),
+			 errmsg("deadlock detected"),
+			 errdetail("%s", buf.data)));
+}
+
+/*
+ * RememberSimpleDeadLock: set up info for DeadLockReport when ProcSleep
+ * detects a trivial (two-way) deadlock.  proc1 wants to block for lockmode
+ * on lock, but proc2 is already waiting and would be blocked by proc1.
+ */
+void
+RememberSimpleDeadLock(PGPROC *proc1,
+					   LOCKMODE lockmode,
+					   LOCK *lock,
+					   PGPROC *proc2)
+{
+	DEADLOCK_INFO *info = &deadlockDetails[0];
+
+	info->locktag = lock->tag;
+	info->lockmode = lockmode;
+	info->pid = proc1->pid;
+	info++;
+	info->locktag = proc2->waitLock->tag;
+	info->lockmode = proc2->waitLockMode;
+	info->pid = proc2->pid;
+	nDeadlockDetails = 2;
+}
